@@ -108,37 +108,36 @@ class BiothingsDumper(HTTPDumper):
                 self._target_backend = self.__class__.TARGET_BACKEND
         return self._target_backend
 
-    @asyncio.coroutine
-    def get_target_backend(self):
+    async def get_target_backend(self):
         """
         Example:
         [{
             'host': 'es6.mygene.info:9200', 
             'index': 'mygene_allspecies_20200823_ufkwdv79', 
+            'index_alias': 'mygene_allspecies', 
             'version': '20200906', 
             'count': 38729977
         }]
         """
-        @asyncio.coroutine
-        def do():
+        async def do():
             cnt = self.target_backend.count()
             return {
                 "host": self.target_backend.target_esidxer.es_host,
                 "index": self.target_backend.target_name,
+                "index_alias": self.target_backend.target_alias,
                 "version": self.target_backend.version,
                 "count": cnt
             }
 
-        return do()
+        result = await do()
+        return result
 
-    @asyncio.coroutine
-    def reset_target_backend(self):
-        @asyncio.coroutine
-        def do():
+    async def reset_target_backend(self):
+        async def do():
             if self.target_backend.target_esidxer.exists_index():
                 self.target_backend.target_esidxer.delete_index()
 
-        return do()
+        await do()
 
     def download(self, remoteurl, localfile, headers={}):
         self.prepare_local_folders(localfile)
@@ -514,8 +513,7 @@ class BiothingsDumper(HTTPDumper):
             if build_meta["metadata"]["repository"]["type"] == "fs":
                 uncompressall(self.new_data_folder)
 
-    @asyncio.coroutine
-    def info(self, version='latest'):
+    async def info(self, version='latest'):
         """
         Display version information (release note, etc...) for given version
         {
@@ -545,8 +543,7 @@ class BiothingsDumper(HTTPDumper):
                         (version, res, res.text))
         return result
 
-    @asyncio.coroutine
-    def versions(self):
+    async def versions(self):
         """
         Display all available versions.
         Example:
