@@ -199,10 +199,31 @@ class DataBuilder(object):
         """
         def no_uploader_running(job_manager):
             """Uploaders could change the data to be merged..."""
-            return len([
-                j for j in job_manager.jobs.values()
-                if j["category"] == UPLOADER_CATEGORY
-            ]) == 0
+            num_offenders = 0
+            # self.sources is not populated. thanks.
+            sources = self.build_config.get("sources", [])
+            offending_sources = set()
+            for src in sources:
+                src_full_name = get_source_fullname(src)
+                offending_sources.add(src_full_name)
+            self.logger.debug("no_uploader_running: src full names %s", offending_sources)
+            for job in job_manager.jobs.values():
+                if job['category'] == UPLOADER_CATEGORY:
+                    if 'source' in job:
+                        if job['source'] in offending_sources:
+                            num_offenders += 1
+                            self.logger.info(
+                                "%s uploader running cannot build for now" % job['source']
+                            )
+                    else:
+                        num_offenders += 1
+                        self.logger.warning(
+                            "uploader with pinfo: %s running, no source info. "
+                            "cannot build for now" % job
+                        )
+                else:
+                    pass  # job is not an uploader
+            return num_offenders == 0
 
         #def no_merger_running():
         #    """
@@ -230,7 +251,9 @@ class DataBuilder(object):
         return pinfo
 
     def setup_log(self):
-        self.logger, _ = get_logger('build_%s' % self.build_name)
+        log_name = self.target_name or self.build_name
+        log_folder = os.path.join(btconfig.LOG_FOLDER, 'build', log_name)
+        self.logger, _ = get_logger('build', log_folder=log_folder, force=True)
 
     def check_ready(self, force=False):
         if force:
@@ -567,6 +590,8 @@ class DataBuilder(object):
         self.target_name = target_name
         self.target_backend.set_target_name(self.target_name)
 
+        self.setup_log()  # Force logs will be stored in the target_name file
+
         self.custom_metadata = {}
         self.clean_old_collections()
         self.logger.info("Merging into target collection '%s'" %
@@ -574,9 +599,7 @@ class DataBuilder(object):
         strargs = "[sources=%s,target_name=%s]" % (sources, target_name)
 
         try:
-
-            @asyncio.coroutine
-            def do():
+            async def do():
                 res = None
                 if "merge" in steps or "post" in steps:
                     job = self.merge_sources(source_names=sources,
@@ -585,7 +608,7 @@ class DataBuilder(object):
                                              job_manager=job_manager,
                                              *args,
                                              **kwargs)
-                    res = yield from job
+                    res = await job
                 if "metadata" in steps:
                     pinfo = self.get_pinfo()
                     pinfo["step"] = "metadata"
@@ -593,7 +616,7 @@ class DataBuilder(object):
                                          transient=True,
                                          init=True,
                                          job={"step": "metadata"})
-                    postjob = yield from job_manager.defer_to_thread(
+                    postjob = await job_manager.defer_to_thread(
                         pinfo,
                         partial(self.store_metadata,
                                 res,
@@ -650,7 +673,7 @@ class DataBuilder(object):
                             raise
 
                     postjob.add_done_callback(stored)
-                    yield from postjob
+                    await postjob
 
             task = asyncio.ensure_future(do())
             return task
@@ -683,8 +706,7 @@ class DataBuilder(object):
                 "Found mapper named '%s' but no mapper associated" %
                 mapper_name)
 
-    @asyncio.coroutine
-    def merge_sources(self,
+    async def merge_sources(self,
                       source_names,
                       steps=["merge", "post"],
                       batch_size=100000,
@@ -726,11 +748,10 @@ class DataBuilder(object):
 
         got_error = False
 
-        @asyncio.coroutine
-        def merge(src_names):
+        async def merge(src_names):
             jobs = []
             for i, src_name in enumerate(src_names):
-                yield from asyncio.sleep(0.0)
+                await asyncio.sleep(0.0)
                 job = self.merge_source(src_name,
                                         batch_size=batch_size,
                                         ids=ids,
@@ -750,12 +771,12 @@ class DataBuilder(object):
                 job.add_done_callback(
                     partial(merged, name=src_name, stats=self.merge_stats))
                 jobs.append(job)
-                yield from asyncio.wait([job])
+                await asyncio.wait([job])
                 # raise error as soon as we know something went wrong
                 if got_error:
                     raise got_error
             tasks = asyncio.gather(*jobs)
-            yield from tasks
+            await tasks
 
         if do_merge:
             if root_sources:
@@ -768,7 +789,7 @@ class DataBuilder(object):
                                      })
                 self.logger.info("Merging root document sources: %s" %
                                  root_sources)
-                yield from merge(root_sources)
+                await merge(root_sources)
                 self.register_status("success",
                                      job={
                                          "step": "merge-root",
@@ -784,7 +805,7 @@ class DataBuilder(object):
                                          "sources": other_sources
                                      })
                 self.logger.info("Merging other resources: %s" % other_sources)
-                yield from merge(other_sources)
+                await merge(other_sources)
                 self.register_status("success",
                                      job={
                                          "step": "merge-others",
@@ -809,7 +830,7 @@ class DataBuilder(object):
                                  job={"step": "post-merge"})
             pinfo = self.get_pinfo()
             pinfo["step"] = "post-merge"
-            job = yield from job_manager.defer_to_thread(
+            job = await job_manager.defer_to_thread(
                 pinfo,
                 partial(self.post_merge, source_names, batch_size,
                         job_manager))
@@ -825,13 +846,13 @@ class DataBuilder(object):
                     got_error = e
 
             job.add_done_callback(postmerged)
-            yield from job
+            await job
             if got_error:
                 raise got_error
         else:
             self.logger.info("Skip post-merge process")
 
-        yield from asyncio.sleep(0.0)
+        await asyncio.sleep(0.0)
         return self.merge_stats
 
     def document_cleaner(self, src_name, *args, **kwargs):
@@ -843,8 +864,7 @@ class DataBuilder(object):
         """
         return None
 
-    @asyncio.coroutine
-    def merge_source(self,
+    async def merge_source(self,
                      src_name,
                      batch_size=100000,
                      ids=None,
@@ -917,7 +937,7 @@ class DataBuilder(object):
             for doc_ids in iter_n(big_doc_ids, batch_size):
                 # try to put some async here to give control back
                 # (but everybody knows it's a blocking call: doc_feeder)
-                yield from asyncio.sleep(0.1)
+                await asyncio.sleep(0.1)
                 cnt += len(doc_ids)
                 pinfo = self.get_pinfo()
                 pinfo["step"] = src_name
@@ -925,7 +945,7 @@ class DataBuilder(object):
                                                             (cnt / total * 100))
                 self.logger.info("Creating merger job #%d/%d, to process '%s' %d/%d (%.1f%%)" %
                                  (bnum, btotal, src_name, cnt, total, (cnt/total*100.)))
-                job = yield from job_manager.defer_to_process(
+                job = await job_manager.defer_to_process(
                     pinfo,
                     partial(merger_worker, self.source_backend[src_name].name,
                             self.target_backend.target_name, doc_ids,
@@ -957,7 +977,7 @@ class DataBuilder(object):
             _ = sum(f.result())
 
         tasks.add_done_callback(done)
-        yield from tasks
+        await tasks
         if got_error:
             raise got_error
         else:
@@ -967,8 +987,6 @@ class DataBuilder(object):
         pass
 
 
-
-
 class LinkDataBuilder(DataBuilder):
     """
     LinkDataBuilder creates a link to the original datasource to be merged, without
@@ -993,8 +1011,7 @@ class LinkDataBuilder(DataBuilder):
         self.target_backend.datasource_name = conf["sources"][0]
         self.target_backend.source_db = self.source_backend
 
-    @asyncio.coroutine
-    def merge_source(self, src_name, *args, **kwargs):
+    async def merge_source(self, src_name, *args, **kwargs):
         total = self.source_backend[src_name].count()
         return {"%s" % src_name: total}
 
@@ -1391,6 +1408,9 @@ class BuilderManager(BaseManager):
                 # TODO: this will get big... but needs to be generic
                 # because we handle different hub db backends (or it needs to be a
                 # specific helper func to be defined all backends
+                # FIXME: this gets slower as hub gets more builds, we are
+                #  finding all builds of all build configs when /whatsnew gets
+                #  requested
                 builds = dbbuild.find({"build_config.name": build_name})
                 builds = sorted(builds, key=lambda e: e["started_at"])
                 if builds:
@@ -1412,6 +1432,20 @@ class BuilderManager(BaseManager):
             for src_name, data in meta_srcs.items():
                 try:
                     srcd = dbdump.find_one({"_id": src_name})
+                    try:
+                        if srcd and srcd.get('download'):
+                            if not srcd['download']['status'] == "success":
+                                srcd = None
+                        if srcd and srcd.get('upload'):
+                            for sub in srcd['upload']['jobs'].values():
+                                if not sub['status'] == "success":
+                                    srcd = None
+                                    break
+                    except KeyError as e:
+                        self.logger.warning(
+                            "whatsnew: src_dump:%s missing keys: %s, not touching",
+                            src_name, e
+                        )
                     if srcd and not srcd.get("download") and srcd.get(
                             "upload"):
                         # this is a collection only source, find all releases in sub sources, hopefully all are the same
@@ -1500,32 +1534,37 @@ class BuilderManager(BaseManager):
         have a different polling schedule.
         """
         # don't use $exists in find(), not all hub backend implements that
-        confs = get_src_build_config().find()
-        autobuild_confs = [conf for conf in confs if "autobuild" in conf]
 
-        @asyncio.coroutine
-        def check_new(conf_name):
+        logger, _ = get_logger('autobuild')
+        schedules = {
+            conf["_id"]: conf["autobuild"]["schedule"]
+            for conf in get_src_build_config().find()
+            if conf.get("autobuild", {}).get("schedule")
+        }
+
+        async def _autobuild(conf_name):
+
             new = self.whatsnew(conf_name)
-            if new[conf_name]["sources"]:
-                new_sources = [(name, info["new"].get("version")) for (name, info) in new[conf_name]["sources"].items()]
-                logging.info("Build configuration '%s' can be launched (autobuid) because" % conf_name
-                             + "some datasources are new: %s" % new_sources)
-                self.merge(conf_name)
-            else:
-                logging.debug("Nothing new for build configuration '%s'" % conf_name)
+            logger.info(f"{conf_name}:{schedules[conf_name]}")
+            logger.info(f"{conf_name}:{new}")
 
-        for conf in autobuild_confs:
+            if new[conf_name]["sources"]:
+                self.merge(conf_name)
+                logger.info(f"{conf_name}:merge(*)")
+            else:
+                logger.info(f"{conf_name}:pass")
+
+        logger.info(datetime.now().astimezone())
+        logger.info(schedules)  # all schedules
+
+        for _id, _schedule in schedules.items():
             try:
-                sch = conf["autobuild"].get("schedule")
-                if not sch:
-                    logging.error("Build configuration needs autobuild but 'schedule' isn't defined" % conf_name)
-                    continue
                 aiocron.crontab(
-                    sch, func=partial(check_new, conf["_id"]),
+                    _schedule, func=partial(_autobuild, _id),
                     start=True, loop=self.job_manager.loop
                 )
-            except Exception as e:
-                logging.exception("Invalid autobuild information for build config '%s': %s" % (conf["_id"], e))
+            except Exception:
+                logger.exception((_id, _schedule))
 
     def trigger_merge(self, doc):
         return self.merge(doc["_id"])
@@ -1614,7 +1653,9 @@ class BuilderManager(BaseManager):
         builds = [b for b in get_src_build().find(q, fields)]
         res = [
             b for b in sorted(
-                builds, key=lambda e: str(e["started_at"]), reverse=True)
+                builds,
+                key=lambda e: str(e.get("started_at") or ""),
+                reverse=True)
         ]
         # set a global status (ie. latest job's status)
         # + get total #docs
