@@ -52,8 +52,11 @@ class BaseSyncer(object):
         self.target_backend = None
         self._meta = None
 
-    def setup_log(self):
-        self.logger, self.logfile = get_logger('sync')
+    def setup_log(self, build_name=None):
+        log_folder = None
+        if build_name:
+            log_folder = os.path.join(btconfig.LOG_FOLDER, "build", build_name)
+        self.logger, self.logfile = get_logger('sync', log_folder=log_folder, force=True)
 
     def get_predicates(self):
         #def no_same_syncer_running(job_manager):
@@ -162,8 +165,7 @@ class BaseSyncer(object):
             self._meta["old"]["backend"]
         return old_db_col_names
 
-    @asyncio.coroutine
-    def sync_cols(self,
+    async def sync_cols(self,
                   diff_folder,
                   batch_size=10000,
                   mode=None,
@@ -191,6 +193,9 @@ class BaseSyncer(object):
         selfcontained = "selfcontained" in self._meta["diff"]["type"]
         old_db_col_names = self.get_target_backend()
         new_db_col_names = self._meta["new"]["backend"]
+
+        self.setup_log(new_db_col_names)
+
         diff_mapping_file = self._meta["diff"]["mapping_file"]
         pinfo = self.get_pinfo()
         self.synced_cols = "%s -> %s" % (old_db_col_names, new_db_col_names)
@@ -220,7 +225,7 @@ class BaseSyncer(object):
                                      transient=True,
                                      init=True,
                                      job={"step": "sync-mapping"})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo, partial(update_mapping))
 
                 def updated(f):
@@ -241,7 +246,7 @@ class BaseSyncer(object):
                         got_error = e
 
                 job.add_done_callback(updated)
-                yield from job
+                await job
 
             if got_error:
                 self.logger.error("Failed to update mapping on index '%s': %s" %
@@ -277,7 +282,7 @@ class BaseSyncer(object):
                     (worker.__name__, diff_file, cnt, total, strwargs))
                 # deepcopy to make we don't embed "self" with unpickleable stuff
                 meta = copy.deepcopy(self._meta)
-                job = yield from self.job_manager.defer_to_process(
+                job = await self.job_manager.defer_to_process(
                     pinfo,
                     partial(worker, diff_file, old_db_col_names,
                             new_db_col_names,
@@ -300,7 +305,7 @@ class BaseSyncer(object):
 
             tasks = asyncio.gather(*jobs)
             tasks.add_done_callback(synced)
-            yield from tasks
+            await tasks
             if got_error:
                 self.logger.error("Failed to sync collection from %s to %s using diff files in '%s': %s" %
                                   (old_db_col_names, new_db_col_names, diff_folder, got_error), extra={"notify": True})
@@ -321,7 +326,7 @@ class BaseSyncer(object):
                 res = indexer.update_mapping_meta({"_meta": new_meta})
                 return res
 
-            job = yield from self.job_manager.defer_to_thread(
+            job = await self.job_manager.defer_to_thread(
                 pinfo, partial(update_metadata))
 
             def updated(f):
@@ -346,7 +351,7 @@ class BaseSyncer(object):
                                  init=True,
                                  job={"step": "sync-meta"})
             job.add_done_callback(updated)
-            yield from job
+            await job
 
             if got_error:
                 self.logger.error("Failed to update metadata on index '%s': %s" %
@@ -355,7 +360,7 @@ class BaseSyncer(object):
 
         if "post" in steps:
             pinfo["step"] = "post"
-            job = yield from self.job_manager.defer_to_thread(
+            job = await self.job_manager.defer_to_thread(
                 pinfo,
                 partial(self.post_sync_cols,
                         diff_folder=diff_folder,
@@ -388,7 +393,7 @@ class BaseSyncer(object):
                                  init=True,
                                  job={"step": "sync-post"})
             job.add_done_callback(posted)
-            yield from job
+            await job
 
             if got_error:
                 self.logger.error("Failed to run post-sync process on index '%s': %s" %
@@ -661,6 +666,7 @@ def sync_es_jsondiff_worker(diff_file,
 
     # delete: remove from "old"
     for ids in iter_n(diff["delete"], batch_size):
+        # FIXME: bulk delete can fail
         del_skip = indexer.delete_docs(ids)
         res["deleted"] += del_skip[0]
         res["skipped"] += del_skip[1]
