@@ -10,11 +10,11 @@ import asyncio
 from functools import partial
 import subprocess
 
-from biothings.utils.mongo import get_previous_collection, get_target_db
-from biothings.utils.hub_db import get_src_build, get_source_fullname
+from .releasenote import ReleaseNoteSrcBuildReader, ReleaseNoteSource
+
+from biothings.utils.mongo import get_previous_collection
+from biothings.utils.hub_db import get_src_build
 import biothings.utils.aws as aws
-from biothings.utils.dataload import update_dict_recur
-from biothings.utils.jsondiff import make as jsondiff
 from biothings.utils.loggers import get_logger
 from biothings.utils.manager import BaseManager, BaseStatusRegisterer
 from biothings.utils.backend import DocMongoBackend
@@ -68,8 +68,11 @@ class BasePublisher(BaseManager, BaseStatusRegisterer):
     def setup(self):
         self.setup_log()
 
-    def setup_log(self):
-        self.logger, self.logfile = get_logger(self.category, self.log_folder)
+    def setup_log(self, build_name=None):
+        log_folder = self.log_folder
+        if build_name:
+            log_folder = os.path.join(btconfig.LOG_FOLDER, "build", build_name)
+        self.logger, self.logfile = get_logger(self.category, log_folder, force=True)
 
     def get_predicates(self):
         return []
@@ -239,9 +242,9 @@ class BasePublisher(BaseManager, BaseStatusRegisterer):
                         return archive_file
                     else:
                         # produce a json file with metadata about the splits
-                        jsonfile = "%s.json" % outfile
+                        jsonfile = "%s.json" % archive_file
                         json.dump({
-                            "filename": outfile,
+                            "filename": archive_file,
                             "parts": flist
                         }, open(jsonfile, "w"))
                         self.logger.info(
@@ -349,7 +352,7 @@ class SnapshotPublisher(BasePublisher):
 
     def get_pre_post_previous_result(self, build_doc, key_value):
         assert build_doc["snapshot"][key_value], "previous step not successful"
-        assert build_doc["snapshot"][key_value]["snapshot"]
+        assert build_doc["snapshot"][key_value]["created_at"]
         previous_result = build_doc["snapshot"][key_value]["conf"]["repository"]
         return previous_result
 
@@ -382,6 +385,9 @@ class SnapshotPublisher(BasePublisher):
                 "Error loading build document using snapshot named '%s': %s" %
                 (snapshot, e))
             raise
+
+        self.setup_log(bdoc["_id"])
+
         if isinstance(bdoc, list):
             raise PublisherException(
                 "Snapshot '%s' found in more than one builds: %s." %
@@ -421,8 +427,7 @@ class SnapshotPublisher(BasePublisher):
         # hold error/exception on each step
         got_error = None
 
-        @asyncio.coroutine
-        def do():
+        async def do():
             jobs = []
             pinfo = self.get_pinfo()
             pinfo["step"] = "publish"
@@ -483,11 +488,11 @@ class SnapshotPublisher(BasePublisher):
                                      publish={"full": {
                                          snapshot: {}
                                      }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo,
                     partial(self.pre_publish, snapshot, self.envconf, bdoc))
                 job.add_done_callback(partial(done, step="pre"))
-                yield from job
+                await job
                 if got_error:
                     raise got_error
                 jobs.append(job)
@@ -624,6 +629,7 @@ class SnapshotPublisher(BasePublisher):
                                             "conf": self.envconf,
                                             "release-note": {
                                                 "err": str(e),
+                                                # TODO: set value to s3basedir in case it not defined before
                                                 "base_dir": s3basedir,
                                                 "bucket": s3_release_bucket
                                             },
@@ -741,11 +747,11 @@ class SnapshotPublisher(BasePublisher):
                                      publish={"fulle": {
                                          snapshot: {}
                                      }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo,
                     partial(self.post_publish, snapshot, self.envconf, bdoc))
                 job.add_done_callback(partial(done, step="post"))
-                yield from job
+                await job
                 jobs.append(job)
 
             def published(f):
@@ -761,11 +767,10 @@ class SnapshotPublisher(BasePublisher):
                         extra={"notify": True})
 
             if jobs:
-                yield from asyncio.wait(jobs)
+                await asyncio.wait(jobs)
                 task = asyncio.gather(*jobs)
                 task.add_done_callback(published)
-                yield from task
-
+                await task
 
         def done(f):
             try:
@@ -854,6 +859,9 @@ class DiffPublisher(BasePublisher):
           * meta: publish/register the version as available for auto-updating hubs
         """
         bdoc = self.load_build(build_name)
+
+        self.setup_log(bdoc["_id"])
+
         assert bdoc, "No such build named '%s'" % build_name
         assert "diff" in bdoc, "No diff release found in build document named '%s'" % build_name
         if previous_build is None:
@@ -896,8 +904,7 @@ class DiffPublisher(BasePublisher):
         # hold error/exception on each step
         got_error = None
 
-        @asyncio.coroutine
-        def do():
+        async def do():
             jobs = []
             pinfo = self.get_pinfo()
             pinfo["source"] = diff_folder
@@ -959,12 +966,12 @@ class DiffPublisher(BasePublisher):
                     publish={"incremental": {
                         previous_build: {}
                     }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo,
                     partial(self.pre_publish, previous_build, self.envconf,
                             bdoc))
                 job.add_done_callback(partial(done, step="pre"))
-                yield from job
+                await job
                 if got_error:
                     raise got_error
                 jobs.append(job)
@@ -985,10 +992,10 @@ class DiffPublisher(BasePublisher):
                     publish={"incremental": {
                         previous_build: {}
                     }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo, partial(self.reset_synced, diff_folder))
                 job.add_done_callback(partial(done, step="reset"))
-                yield from job
+                await job
                 if got_error:
                     raise got_error
                 jobs.append(job)
@@ -1008,7 +1015,7 @@ class DiffPublisher(BasePublisher):
                     publish={"incremental": {
                         previous_build: {}
                     }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo,
                     partial(aws.send_s3_folder,
                             diff_folder,
@@ -1020,7 +1027,7 @@ class DiffPublisher(BasePublisher):
                             s3_bucket=s3_diff_bucket,
                             overwrite=True))
                 job.add_done_callback(partial(done, step="upload"))
-                yield from job
+                await job
                 jobs.append(job)
 
             if "meta" in steps:
@@ -1193,12 +1200,12 @@ class DiffPublisher(BasePublisher):
                     publish={"incremental": {
                         previous_build: {}
                     }})
-                job = yield from self.job_manager.defer_to_thread(
+                job = await self.job_manager.defer_to_thread(
                     pinfo,
                     partial(self.post_publish, previous_build, self.envconf,
                             bdoc))
                 job.add_done_callback(partial(done, step="post"))
-                yield from job
+                await job
                 if got_error:
                     raise got_error
                 jobs.append(job)
@@ -1215,10 +1222,10 @@ class DiffPublisher(BasePublisher):
                         % (diff_folder, e),
                         extra={"notify": True})
 
-            yield from asyncio.wait(jobs)
+            await asyncio.wait(jobs)
             task = asyncio.gather(*jobs)
             task.add_done_callback(uploaded)
-            yield from task
+            await task
 
         def done(f):
             try:
@@ -1283,9 +1290,12 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
     def setup(self):
         self.setup_log()
 
-    def setup_log(self):
-        self.logger, self.logfile = get_logger(RELEASEMANAGER_CATEGORY,
-                                               self.log_folder)
+    def setup_log(self, build_name=None):
+        name = RELEASEMANAGER_CATEGORY
+        log_folder = self.log_folder
+        if build_name:
+            log_folder = os.path.join(btconfig.LOG_FOLDER, "build", build_name)
+        self.logger, self.logfile = get_logger(name, log_folder=log_folder, force=True)
 
     def poll(self, state, func):
         super().poll(state, func, col=get_src_build())
@@ -1392,9 +1402,8 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
 
     def load_build(self, key_name, stage=None):
         if stage is None:
-            # picke build doc searching for snapshot then diff key if not found
-            return self.load_doc(key_name, "snapshot") or self.load_doc(
-                key_name, "diff")
+            # pick build doc searching for snapshot then diff key if not found
+            return self.load_doc(key_name, "snapshot") or self.load_doc(key_name, "diff")
         else:
             return self.load_doc(key_name, stage)
 
@@ -1457,9 +1466,9 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
                 "snapshot", {}):
             # doc was returend with snapshot_or_build_name matching _id, not snapshot name, invalidate
             snapshot_doc = None
-        # TODO 
+        # TODO
         # diff is wrt another diff, maybe should be this way?
-        #-------------------------------------- 
+        #--------------------------------------
         # if diff_doc and snapshot_or_build_name not in diff_doc.get("diff", {}):
         if diff_doc and not diff_doc.get("diff", {}):
             diff_doc = None
@@ -1512,10 +1521,27 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
         return content
 
     def create_release_note_from_build(self, build_doc):
-        @asyncio.coroutine
-        def _():
-            yield from self.create_release_note(old=None, new=build_doc["_id"])
-            # release note created, now we consider what's next
+        async def _():
+            if build_doc.get("release_note"):
+                self.logger.info(
+                    "Not a brand-new build. "
+                    "Skip release note automation.")
+                return
+            try:
+                old = get_previous_collection(build_doc["_id"])
+            except AssertionError:
+                self.logger.warning(
+                    "Cannot find the previous build. "
+                    "Create a fresh release note.")
+                old = "none"
+
+            if old == build_doc["_id"]:
+                self.logger.error(
+                    "Error finding the previous build. "
+                    "Skip release note automation. ")
+                return
+            await self.create_release_note(old=old, new=build_doc["_id"])
+
             build_conf = AutoBuildConfig(build_doc['build_config'])
             if build_conf.should_publish_new_diff() or build_conf.should_publish_new_snapshot():
                 # TODO differentiate at some level
@@ -1541,61 +1567,68 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
 
         txt 'format' is the only one supported for now.
         """
-        bdoc = self.load_build(new)
+        build_doc = self.load_build(new)
+
+        self.setup_log(build_doc["_id"])
+
         old = old or get_previous_collection(new)
+
         release_folder = generate_folder(btconfig.RELEASE_PATH, old, new)
         if not os.path.exists(release_folder):
             os.makedirs(release_folder)
+
         filepath = None
 
         def do():
-            changes = self.build_release_note(old, new, note=note)
+            release_note_source = self.build_release_note(old, new, note=note)
+            changes = release_note_source.to_dict()
+
+            assert format == "txt", "Only 'txt' format supported for now"
+
             nonlocal filepath
             nonlocal filename
-            assert format == "txt", "Only 'txt' format supported for now"
-            filename = filename or "release_%s.%s" % (
-                changes["new"]["_version"], format)
+
+            filename = filename or "release_%s.%s" % (changes["new"]["_version"], format)
             filepath = os.path.join(release_folder, filename)
-            render = ReleaseNoteTxt(changes)
+            render = ReleaseNoteTxt(release_note_source)
             txt = render.save(filepath)
+
             filename = filename.replace(".%s" % format, ".json")
             filepath = os.path.join(release_folder, filename)
             json.dump(changes, open(filepath, "w"), indent=True)
+
             return {"txt": txt, "changes": changes}
 
-        @asyncio.coroutine
-        def main(release_folder):
+        async def main(release_folder):
             got_error = False
             pinfo = self.get_pinfo()
             pinfo["step"] = "release_note"
             pinfo["source"] = release_folder
             pinfo["description"] = filename
-            self.register_status(bdoc,
+            self.register_status(build_doc,
                                  "release_note",
                                  "generating",
                                  transient=True,
                                  init=True,
                                  job={"step": "release_note"},
                                  release_note={old: {}})
-            job = yield from self.job_manager.defer_to_thread(pinfo, do)
+            job = await self.job_manager.defer_to_thread(pinfo, do)
 
             def reported(f):
                 nonlocal got_error
                 try:
                     res = f.result()
-                    self.register_status(bdoc,
+                    self.register_status(build_doc,
                                          "release_note",
                                          "success",
                                          job={"step": "release_note"},
                                          release_note={
                                              old: {
                                                  "changes": res["changes"],
-                                                 "release_folder":
-                                                 release_folder
+                                                 "release_folder": release_folder
                                              }
                                          })
-                    self.logger.info("Release note ready, saved in %s: %s" %
-                                     (release_folder, res["txt"]),
+                    self.logger.info("Release note ready, saved in %s: %s" % (release_folder, res["txt"]),
                                      extra={"notify": True})
                     set_pending_to_publish(new)
                 except Exception as e:
@@ -1603,166 +1636,82 @@ class ReleaseManager(BaseManager, BaseStatusRegisterer):
                     got_error = e
 
             job.add_done_callback(reported)
-            yield from job
+            await job
             if got_error:
-                self.logger.exception("Failed to create release note: %s" %
-                                      got_error,
+                self.logger.exception("Failed to create release note: %s" % got_error,
                                       extra={"notify": True})
-                self.register_status(bdoc,
+                self.register_status(build_doc,
                                      "release_note",
                                      "failed",
-                                     job={
-                                         "step": "release_note",
-                                         "err": str(e)
-                                     },
+                                     job={"step": "release_note", "err": str(got_error)},
                                      release_note={old: {}})
                 raise got_error
 
         job = asyncio.ensure_future(main(release_folder))
         return job
 
-    def build_release_note(self, old_colname, new_colname, note=None):
+    def build_release_note(self, old_colname, new_colname, note=None) -> ReleaseNoteSource:
         """
         Build a release note containing most significant changes between build names "old_colname" and "new_colname".
         An optional end note can be added to bring more specific information about the release.
 
-        Return a dictionnary containing significant changes.
+        Return a dictionary containing significant changes.
         """
-        def get_counts(dstats):
-            stats = {}
-            for subsrc, count in dstats.items():
-                try:
-                    src_sub = get_source_fullname(subsrc).split(".")
-                except AttributeError:
-                    # not a merge stats coming from a source
-                    # (could be custom field stats, eg. total_* in mygene)
-                    src_sub = [subsrc]
-                if len(src_sub) > 1:
-                    # we have sub-sources we need to split the count
-                    src, sub = src_sub
-                    stats.setdefault(src, {})
-                    stats[src][sub] = {"_count": count}
-                else:
-                    src = src_sub[0]
-                    stats[src] = {"_count": count}
-            return stats
+        def get_backend_from_colname(colname):
+            backend = create_backend(colname)
+            assert isinstance(backend, DocMongoBackend), \
+                "Only MongoDB backend types are allowed when generating a release note"
+            assert backend.target_collection.database.name == btconfig.DATA_TARGET_DATABASE, \
+                "Target databases must match current DATA_TARGET_DATABASE setting"
+            return backend
 
-        def get_versions(doc):
-            try:
-                versions = dict((k, {"_version": v["version"]}) for k, v in
-                                doc.get("_meta", {}).get("src", {}).items() if "version" in v)
-            except KeyError:
-                # previous version format
-                versions = dict((k, {
-                    "_version": v
-                })
-                    for k, v in doc.get("_meta", {}).get(
-                        "src_version", {}).items())
-            return versions
+        def check_cold_collection(src_build_reader: ReleaseNoteSrcBuildReader):
+            if src_build_reader.has_cold_collection():
+                # We have been querying src_build with collection names across publisher.py,
+                #   because src_build `_id`s and collection names are interchangeable.
+                # See https://github.com/biothings/biothings.api/blob/master/biothings/hub/databuild/builder.py#L311
+                cold_doc = get_src_build().find_one({"_id": src_build_reader.cold_collection_name})
+                cold_reader = ReleaseNoteSrcBuildReader(cold_doc)
 
-        diff_folder = generate_folder(btconfig.DIFF_PATH, old_colname,
-                                      new_colname)
+                src_build_reader.attach_cold_src_build_reader(cold_reader)
+
+            return src_build_reader
+
+        diff_folder = generate_folder(btconfig.DIFF_PATH, old_colname, new_colname)
         try:
             metafile = os.path.join(diff_folder, "metadata.json")
             metadata = json.load(open(metafile))
+
             old_colname = metadata["old"]["backend"]
             new_colname = metadata["new"]["backend"]
             diff_stats = metadata["diff"]["stats"]
         except FileNotFoundError:
             # we're generating a release note without diff information
-            self.logger.info(
-                "No metadata.json file found, this release note won't have diff stats included"
-            )
+            self.logger.info("No metadata.json file found, this release note won't have diff stats included")
             diff_stats = {}
 
-        new = create_backend(new_colname)
-        old = create_backend(old_colname)
-        assert isinstance(old, DocMongoBackend) and isinstance(new, DocMongoBackend), \
-            "Only MongoDB backend types are allowed when generating a release note"
-        assert old.target_collection.database.name == btconfig.DATA_TARGET_DATABASE and \
-            new.target_collection.database.name == btconfig.DATA_TARGET_DATABASE, \
-            "Target databases must match current DATA_TARGET_DATABASE setting"
-        new_doc = get_src_build().find_one({"_id": new.target_collection.name})
-        if not new_doc:
+        new_backend = get_backend_from_colname(new_colname)
+        old_backend = get_backend_from_colname(old_colname)
+
+        new_src_build_doc = get_src_build().find_one({"_id": new_backend.target_collection.name})
+        if not new_src_build_doc:
             raise PublisherException("Collection '%s' has no corresponding build document" %
-                                     new.target_collection.name)
-        # old_doc doesn't have to exist (but new_doc has) in case we build a initial release note
+                                     new_backend.target_collection.name)
+        # old_doc doesn't have to exist (but new_src_build_doc has) in case we build a initial release note
         # compared against nothing
-        old_doc = get_src_build().find_one({"_id": old.target_collection.name
-                                            }) or {}
-        tgt_db = get_target_db()
-        old_total = tgt_db[old.target_collection.name].count()
-        new_total = tgt_db[new.target_collection.name].count()
-        changes = {
-            "old": {
-                "_version": old.version,
-                "_count": old_total,
-            },
-            "new": {
-                "_version": new.version,
-                "_count": new_total,
-                "_fields": {},
-                "_summary": diff_stats,
-            },
-            "stats": {
-                "added": {},
-                "deleted": {},
-                "updated": {},
-            },
-            "note": note,
-            "generated_on": str(datetime.now().astimezone()),
-            "sources": {
-                "added": {},
-                "deleted": {},
-                "updated": {},
-            }
-        }
-        # for later use
-        new_versions = get_versions(new_doc)
-        old_versions = get_versions(old_doc)
-        # now deal with stats/counts. Counts are related to uploader, ie. sub-sources
-        new_merge_stats = get_counts(new_doc.get("merge_stats", {}))
-        old_merge_stats = get_counts(old_doc.get("merge_stats", {}))
-        new_stats = get_counts(new_doc.get("_meta", {}).get("stats", {}))
-        old_stats = get_counts(old_doc.get("_meta", {}).get("stats", {}))
-        new_info = update_dict_recur(new_versions, new_merge_stats)
-        old_info = update_dict_recur(old_versions, old_merge_stats)
+        old_src_build_doc = get_src_build().find_one({"_id": old_backend.target_collection.name}) or {}
 
-        def analyze_diff(ops, destdict, old, new):
-            for op in ops:
-                # get main source / main field
-                key = op["path"].strip("/").split("/")[0]
-                if op["op"] == "add":
-                    destdict["added"][key] = new[key]
-                elif op["op"] == "remove":
-                    destdict["deleted"][key] = old[key]
-                elif op["op"] == "replace":
-                    destdict["updated"][key] = {
-                        "new": new[key],
-                        "old": old[key]
-                    }
-                else:
-                    raise ValueError(
-                        "Unknown operation '%s' while computing changes" %
-                        op["op"])
+        new_src_build_reader = ReleaseNoteSrcBuildReader(new_src_build_doc)
+        new_src_build_reader = check_cold_collection(new_src_build_reader)
+        old_src_build_reader = ReleaseNoteSrcBuildReader(old_src_build_doc)
+        old_src_build_reader = check_cold_collection(old_src_build_reader)
 
-        # diff source info
-        # this only works on main source information: if there's a difference in a
-        # sub-source, it won't be shown but considered as if it was the main-source
-        ops = jsondiff(old_info, new_info)
-        analyze_diff(ops, changes["sources"], old_info, new_info)
+        release_note_source = ReleaseNoteSource(old_src_build_reader, new_src_build_reader,
+                                                diff_stats_from_metadata_file=diff_stats, addon_note=note)
 
-        ops = jsondiff(old_stats, new_stats)
-        analyze_diff(ops, changes["stats"], old_stats, new_stats)
-
-        # mapping diff: we re-compute them and don't use any mapping.pyobj because that file
-        # only allows "add" operation as a safety rule (can't delete fields in ES mapping once indexed)
-        ops = jsondiff(old_doc.get("mapping", {}), new_doc["mapping"])
-        for op in ops:
-            changes["new"]["_fields"].setdefault(op["op"], []).append(
-                op["path"].strip("/").replace("/", "."))
-
-        return changes
+        changes = release_note_source.to_dict()
+        self.logger.debug(f"old_colname={old_colname}, new_colname={new_colname}, changes={changes}")
+        return release_note_source
 
     def release_info(self, env=None, remote=False):
         res = copy.deepcopy(self.release_config)
