@@ -65,13 +65,12 @@ class AutoHubFeature(object):
 
         return vurls
 
-    def install(self, src_name, version="latest", dry=False, force=False):
+    def install(self, src_name, version="latest", dry=False, force=False, use_no_downtime_method=True):
         """
         Update hub's data up to the given version (default is latest available),
         using full and incremental updates to get up to that given version (if possible).
         """
-        @asyncio.coroutine
-        def do(version):
+        async def do(version):
             try:
                 dklass = self.managers["dump_manager"][src_name][0]   # only one dumper allowed / source
                 dobj = self.managers["dump_manager"].create_instance(dklass)
@@ -91,14 +90,17 @@ class AutoHubFeature(object):
                     logging.info("Downloading data for version '%s'", step_version)
                     jobs = self.managers["dump_manager"].dump_src(src_name, version=step_version, force=force)
                     download = asyncio.gather(*jobs)
-                    res = yield from download
+                    res = await download
                     assert len(res) == 1
                     if res[0] is None:
                         # download ready, now install
                         logging.info("Updating backend to version '%s'", step_version)
-                        jobs = self.managers["upload_manager"].upload_src(src_name)
+                        jobs = self.managers["upload_manager"].upload_src(
+                            src_name,
+                            use_no_downtime_method=use_no_downtime_method,
+                        )
                         upload = asyncio.gather(*jobs)
-                        res = yield from upload
+                        res = await upload
 
             except Exception:
                 self.logger.exception("data install failed")
@@ -183,9 +185,17 @@ class AutoHubFeature(object):
     def configure_auto_release(self, config):
         if hasattr(config, "AUTO_RELEASE_CONFIG"):
             if isinstance(config.AUTO_RELEASE_CONFIG, dict):
-                for src in config.AUTO_RELEASE_CONFIG:
+                for src, src_config in config.AUTO_RELEASE_CONFIG.items():
+                    schedule = src_config
+                    install_args = {
+                        "src_name": src
+                    }
+                    if isinstance(src_config, dict):
+                        schedule = src_config["schedule"]
+                        install_args.update(src_config.get("extra") or {})
+
                     self.logger.info("Scheduling auto release for %s.", src)
                     self.managers["job_manager"].submit(
-                        partial(self.install, src),
-                        config.AUTO_RELEASE_CONFIG[src]
+                        partial(self.install, **install_args),
+                        schedule,
                     )
