@@ -2,6 +2,7 @@
 from functools import partial
 
 from elasticsearch.exceptions import NotFoundError, TransportError
+from pymongo import UpdateOne
 
 from biothings.utils.es import ESIndexer
 from biothings import config as btconfig
@@ -44,6 +45,7 @@ class DocBackendBase(object):
            Final optimization or compacting can be done here as well.
         '''
 
+
 class DocMemoryBackend(DocBackendBase):
     name = 'memory'
 
@@ -80,6 +82,7 @@ class DocMemoryBackend(DocBackendBase):
         from biothings.utils.common import dump
         dump(self.target_dict, self.target_name + '.pyobj')
 
+
 class DocMongoBackend(DocBackendBase):
     name = 'mongo'
 
@@ -97,8 +100,8 @@ class DocMongoBackend(DocBackendBase):
         if not isinstance(other, DocMongoBackend):
             return False
         return self.target_name == other.target_name and \
-               self.target_collection.database.name == other.target_collection.database.name and \
-               self.target_collection.database.client.address == other.target_collection.database.client.address
+            self.target_collection.database.name == other.target_collection.database.name and \
+            self.target_collection.database.client.address == other.target_collection.database.client.address
 
     @property
     def target_name(self):
@@ -145,23 +148,18 @@ class DocMongoBackend(DocBackendBase):
         '''if id does not exist in the target_collection,
             the update will be ignored except if upsert is True
         '''
-        bulk = self.target_collection.initialize_ordered_bulk_op()
-        at_least_one = False
+        bulk = []
         for doc in docs:
-            at_least_one = True
-            op = bulk.find({'_id': doc["_id"]})
-            if upsert:
-                op = op.upsert()
-            op.update({"$set": doc})
-        if at_least_one:
-            res = bulk.execute()
+            bulk.append(UpdateOne({'_id': doc["_id"]}, {"$set": doc}, upsert=upsert))
+
+        if bulk:
+            result = self.target_collection.bulk_write(bulk)
             # if doc is the same, it'll be matched but not modified.
             # but for us, it's been processed. if upserted, then it can't be matched
-            # before (so matched cound doesn't include upserted). finally, it's only update
-            # ops, so don't count nInserted and nRemoved
-            return res["nMatched"] + res["nUpserted"]
-        else:
-            return 0
+            # before (so matched count doesn't include upserted). finally, it's only update
+            # ops, so don't count inserted_count and deleted_count
+            return result.matched_count + result.upserted_count
+        return 0
 
     def update_diff(self, diff, extra={}):
         '''update a doc based on the diff returned from diff.diff_doc
@@ -210,7 +208,7 @@ class DocMongoBackend(DocBackendBase):
         total_cnt = 0
         for i in range(0, len(ids), step):
             _ids = ids[i:i + step]
-            _cnt = self.target_collection.find({'_id': {'$in': _ids}}).count()
+            _cnt = self.target_collection.count_documents({'_id': {'$in': _ids}})
             total_cnt += _cnt
         return total_cnt
 
@@ -227,9 +225,11 @@ class DocMongoBackend(DocBackendBase):
             deleted += res.deleted_count
         return deleted
 
+
 # backward-compatible
 DocMongoDBBackend = DocMongoBackend
 
+
 class DocESBackend(DocBackendBase):
     name = 'es'
 
@@ -239,7 +239,7 @@ class DocESBackend(DocBackendBase):
             self._target_esidxer_provider = esidxer
             self._target_esidxer = None
         else:
-            _target_esidxer_provider = None
+            self._target_esidxer_provider = None
             self._target_esidxer = esidxer
 
     @property
@@ -252,6 +252,14 @@ class DocESBackend(DocBackendBase):
     def target_name(self):
         return self.target_esidxer._index
 
+    @property
+    def target_alias(self):
+        try:
+            alias_info = self.target_esidxer.get_alias(self.target_name) or {}
+            return list(alias_info[self.target_name]["aliases"].keys())[0]
+        except Exception:
+            return
+
     @property
     def version(self):
         try:
@@ -327,6 +335,7 @@ class DocESBackend(DocBackendBase):
             raise Exception("Cannot create backend class from options, ensure that es_index, es_host, and es_doc_type are set")
         return cls(ESIndexer(index=options.es_index, doc_type=options.es_doc_type, es_host=options.es_host))
 
+
 class DocBackendOptions(object):
     def __init__(self, cls, es_index=None, es_host=None, es_doc_type=None,
                  mongo_target_db=None, mongo_target_collection=None):
