@@ -2,29 +2,31 @@
 This module contains util functions may be shared by both BioThings data-hub and web components.
 In general, do not include utils depending on any third-party modules.
 """
+import asyncio
 import base64
+import glob
+import gzip
+import hashlib
+import importlib
+import inspect
+import io
+import json
+import logging
 import os
 import os.path
-import io
+import pickle
 import random
 import string
 import sys
 import time
-from itertools import islice
-from contextlib import contextmanager
-from shlex import shlex
-import pickle
-import json
-import logging
-import importlib
-import hashlib
-import asyncio
-import inspect
 import types
-import gzip
-import glob
+from collections import UserDict, UserList
+from contextlib import contextmanager
 from datetime import date, datetime, timezone
 from functools import partial
+from itertools import islice
+from shlex import shlex
+
 # from json serial, catching special type
 # import _sre     # TODO: unused import;remove it once confirmed
 
@@ -342,6 +344,7 @@ def dump(obj, filename, protocol=2, compress='gzip'):
 def dump2gridfs(obj, filename, db, protocol=2):
     '''Save a compressed (support gzip only) object to MongoDB gridfs.'''
     import gridfs
+
     # import gzip
     fs = gridfs.GridFS(db)
     if fs.exists(_id=filename):
@@ -566,16 +569,20 @@ def newer(t0, t1, fmt='%Y%m%d'):
     return datetime.strptime(t0, fmt) < datetime.strptime(t1, fmt)
 
 
-class DateTimeJSONEncoder(json.JSONEncoder):
+class BiothingsJSONEncoder(json.JSONEncoder):
     '''A class to dump Python Datetime object.
         json.dumps(data, cls=DateTimeJSONEncoder, indent=indent)
     '''
 
     def default(self, o):
-        if isinstance(o, datetime):
+        if isinstance(o, (datetime, date)):
             return o.isoformat()
+        elif isinstance(o, UserDict):
+            return dict(o)
+        elif isinstance(o, UserList):
+            return list(o)
         else:
-            return super(DateTimeJSONEncoder, self).default(o)
+            return super().default(o)
 
 
 # https://stackoverflow.com/questions/11875770/how-to-overcome-datetime-datetime-not-json-serializable
@@ -708,8 +715,8 @@ def gunzip(f, pattern="*.gz"):
         logging.info("Done gunzip '%s'", gz.name)
     fout.close()
 
-@asyncio.coroutine
-def aiogunzipall(folder, pattern, job_manager, pinfo):
+
+async def aiogunzipall(folder, pattern, job_manager, pinfo):
     """
     Gunzip all files in folder matching pattern. job_manager is used
     for parallelisation, and pinfo is a pre-filled dict used by
@@ -720,8 +727,9 @@ def aiogunzipall(folder, pattern, job_manager, pinfo):
     logging.info("Unzipping files in '%s'", folder)
     for f in glob.glob(os.path.join(folder, pattern)):
         pinfo["description"] = os.path.basename(f)
-        suffix = pattern.replace("*", "")
-        job = yield from job_manager.defer_to_process(pinfo, partial(gunzip, f, suffix=suffix))
+        job = await job_manager.defer_to_process(
+            pinfo, partial(gunzip, f, pattern=pattern)
+        )
 
         def gunzipped(fut, infile):
             try:
@@ -736,7 +744,7 @@ def aiogunzipall(folder, pattern, job_manager, pinfo):
         if got_error:
             raise got_error
     if jobs:
-        yield from asyncio.gather(*jobs)
+        await asyncio.gather(*jobs)
         if got_error:
             raise got_error
 
@@ -797,10 +805,10 @@ def traverse(obj, leaf_node=False):
         ('snpeff.ann.feature_id', 'NM_001256678.1')
     )
     """
-    if isinstance(obj, dict):  # path level increases
+    if isinstance(obj, (dict, UserDict)):  # path level increases
         for key in obj:
             for sub_path, val in traverse(obj[key], leaf_node):
-                yield '.'.join((key, sub_path)).strip('.'), val
+                yield '.'.join((str(key), str(sub_path))).strip('.'), val
         if not leaf_node:
             yield '', obj
     elif isinstance(obj, list):  # does not affect path
@@ -811,3 +819,61 @@ def traverse(obj, leaf_node=False):
             yield '', obj
     elif leaf_node:  # including str, int, float, and *None*.
         yield '', obj
+
+def run_once():
+    """
+    should_run_task_1 = run_once()
+    print(should_run_task_1()) -> True
+    print(should_run_task_1()) -> False
+    print(should_run_task_1()) -> False
+    print(should_run_task_1()) -> False
+
+    should_run_task_2 = run_once()
+    print(should_run_task_2('2a')) -> True
+    print(should_run_task_2('2b')) -> True
+    print(should_run_task_2('2a')) -> False
+    print(should_run_task_2('2b')) -> False
+    ...
+    """
+
+    has_run = set()
+
+    def should_run(identifier=None):
+
+        if identifier in has_run:
+            return False
+
+        has_run.add(identifier)
+        return True
+
+    return should_run
+
+def merge(x, dx):
+    """
+    Merge dictionary dx (Δx) into dictionary x.
+    If __REPLACE__ key is present in any level z in dx,
+    z in x is replaced, instead of merged, with z in dx.
+    """
+    assert isinstance(x, dict)
+    assert isinstance(dx, dict)
+
+    if dx.pop("__REPLACE__", None):
+        # merge dx with "nothing" just to
+        # make sure to remove any "__REPLACE__"
+        _y = {}
+        merge(_y, dx)
+        x.clear()
+        x.update(_y)
+        return x
+
+    for k, v in dx.items():
+        if isinstance(v, dict):
+            if v.get("__REMOVE__"):
+                x.pop(k, None)
+                continue
+            if not isinstance(x.get(k), dict):
+                x[k] = {}
+            merge(x[k], v)
+        else:
+            x[k] = v
+    return x
