@@ -9,7 +9,8 @@ import csv
 import os
 import os.path
 import json
-import collections
+from collections.abc import Mapping
+from collections import OrderedDict
 from functools import total_ordering
 
 from .common import open_anyfile, is_str, safewfile, anyfile
@@ -122,7 +123,7 @@ def boolean_convert(d, convert_keys=None, level=0):
                     d[key] = [boolean_convert(v, convert_keys, level+1) for v in val]
                 else:
                     d[key] = [to_boolean(x) for x in val]
-            elif isinstance(val, dict) or isinstance(val, collections.OrderedDict):
+            elif isinstance(val, dict) or isinstance(val, OrderedDict):
                 d[key] = boolean_convert(val, convert_keys, level+1)
             else:
                 d[key] = to_boolean(val)
@@ -422,7 +423,7 @@ def tabfile_tester(datafile, header=1, sep='\t'):
         for ld in reader:
             del ld
             lineno += 1
-    except:
+    except Exception:
         print("Error at line number:", lineno)
         raise
 
@@ -597,6 +598,8 @@ class MinType(object):
 
     def __eq__(self, other):
         return self is other
+
+
 Min = MinType()
 
 
@@ -607,6 +610,8 @@ def traverse_keys(d, include_keys=None, exclude_keys=None):
     If include_keys is specified, only traverse the list from include_kes a.b, a.b.c
     If exclude_keys is specified, only exclude the list from exclude_keys
 
+    if a key in include_keys/exclude_keys is not found in d, it's skipped quietly.
+
     :param d: a dictionary to traverse keys on
     :param include_keys: only traverse these keys (optional)
     :param exclude_keys: exclude all other keys except these keys (optional)
@@ -628,7 +633,13 @@ def traverse_keys(d, include_keys=None, exclude_keys=None):
     if include_keys:
         for k in include_keys:
             for val in key_value(d, k):
-                yield k, val
+                if val:
+                    # only yield non-empty value
+                    # when val is None, it could be either:
+                    #   1. k is not found in d
+                    #   2. the value of k in d is indeed None
+                    # For now, we cannot tell which case, just skip it
+                    yield k, val
     else:
         for kl, val in traverse_helper(d, []):
             key = '.'.join(kl)
@@ -720,7 +731,7 @@ def update_dict_recur(d, u):
     (so existing values in d but not in u are kept even if nested)
     """
     for k, v in u.items():
-        if isinstance(v, collections.Mapping):
+        if isinstance(v, Mapping):
             r = update_dict_recur(d.get(k, {}), v)
             d[k] = r
         else:
@@ -992,13 +1003,17 @@ def dict_traverse(d, func, traverse_list=False):
         items = sorted(d.items(), key=lambda x: x[0])
     except TypeError:
         # not sortable
-        items = d.items()
+        # need to make a copy first, because d will be updated during
+        #   the iteration, a RuntimeError will be raised otherwise:
+        #   RuntimeError: dictionary keys changed during iteration
+        items = d.copy().items()
     for k, v in items:
         if isinstance(v, dict):
             dict_traverse(v, func, traverse_list=traverse_list)
         elif traverse_list and isinstance(v, list):
             for e in v:
-                dict_traverse(e, func, traverse_list=traverse_list)
+                if isinstance(e, dict):
+                    dict_traverse(e, func, traverse_list=traverse_list)
         else:
             newk, newv = func(k, v)
             d.pop(k)
