@@ -1,12 +1,16 @@
-import time, logging, os, io, glob, datetime
+import time, logging, os, io, glob, datetime, inspect
 import dateutil.parser as dtparser
+from collections.abc import Iterable
 from functools import wraps
 from pymongo import MongoClient, DESCENDING
-from pymongo.collection import Collection
+from pymongo.database import Database as PymongoDatabase
+from pymongo.collection import Collection as PymongoCollection
+from pymongo.errors import AutoReconnect
 from functools import partial
 from collections import defaultdict
 import bson
 
+
 from biothings.utils.common import timesofar, get_random_string, iter_n, \
                                    open_compressed_file, get_compressed_outfile, \
                                    dotdict
@@ -16,6 +20,51 @@ from biothings.utils.hub_db import IDatabase, ChangeWatcher
 config = None
 
 
+def handle_autoreconnect(cls_instance, func):
+    """After upgrading the pymongo package from 3.12 to 4.x,
+    the AutoReconnect: "connection pool paused" problem appears quite often.
+    It is not clear that the problem happens with our codebase, maybe a pymongo's problem.
+
+    This function is an attempt to handle the AutoReconnect exception, without modifying our codebase.
+    When the exception is raised, we just wait for some time, then retry.
+    If the error still happens after MAX_RETRY, it must be a connection-related problem.
+    We should stop retrying and raise error.
+
+    Ref: https://github.com/newgene/biothings.api/pull/40#issuecomment-1185334545
+    """
+
+    MAX_RETRY = 30
+    SLEEP_TIME = 0.5  # seconds
+
+    def inner(*args, **kwargs):
+        retry = 0
+        while retry < MAX_RETRY:
+            try:
+                return func(*args, **kwargs)
+            except AutoReconnect as ex:
+                retry += 1
+                time.sleep(SLEEP_TIME)
+
+        raise MaxRetryAutoReconnectException()
+    return inner
+
+
+class HandleAutoReconnectMixin:
+    """This mixin will decor any non-hidden method with handle_autoreconnect decorator"""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        for name, func in inspect.getmembers(self, inspect.ismethod):
+            if name.startswith("_"):
+                continue
+            setattr(self, name, handle_autoreconnect(self, func))
+
+
+class MaxRetryAutoReconnectException(AutoReconnect):
+    """Raised when we reach maximum retry to connect to Mongo server"""
+
+
 class DummyCollection(dotdict):
     def count(self):
         return None
@@ -32,11 +81,63 @@ class DummyDatabase(dotdict):
         return DummyCollection()
 
 
-class Database(MongoClient,IDatabase):
+class Collection(HandleAutoReconnectMixin, PymongoCollection):
+    # https://pymongo.readthedocs.io/en/4.1.1/migrate-to-pymongo4.html
+
+    def __bool__(self):
+        return self is not None
+
+    def insert(self, doc_or_docs, *args, **kwargs):
+        if isinstance(doc_or_docs, Iterable):
+            return self.insert_many(doc_or_docs)
+        else:
+            return self.insert_one(doc_or_docs)
+
+    def update(self, spec, doc, *args, **kwargs):
+        if kwargs.pop("multi", None):
+            return self.update_many(spec, doc, *args, **kwargs)
+        else:
+            return self.update_one(spec, doc, *args, **kwargs)
+
+    def remove(self, spec_or_id=None, **kwargs):
+        if kwargs.pop("multi", None):
+            return self.delete_many(spec_or_id, **kwargs)
+        else:
+            return self.delete_one(spec_or_id, **kwargs)
+
+    def save(self, doc, *args, **kwargs):
+        if '_id' in doc:
+            kwargs["upsert"] = True
+            self.replace_one({'_id': doc['_id']}, doc, *args, **kwargs)
+            return doc['_id']
+        else:
+            res = self.insert_one(doc, *args, **kwargs)
+            return res.inserted_id
+
+    def count(self, _filter=None, **kwargs):
+        if _filter:
+            return self.count_documents(_filter, **kwargs)
+        return self.estimated_document_count(**kwargs)
+
+
+class Database(HandleAutoReconnectMixin, PymongoDatabase):
+    def __bool__(self):
+        return self is not None
+
+    def __getitem__(self, name):
+        return Collection(self, name)
+
+    def collection_names(self, include_system_collections=True, session=None):
+        _filter = None
+        if not include_system_collections:
+            _filter = {"name": {"$regex": r"^(?!system\\.)"}}
+        return self.list_collection_names(session=session, filter=_filter)
+
+
+class DatabaseClient(HandleAutoReconnectMixin, MongoClient, IDatabase):
+    def __getitem__(self, name):
+        return Database(self, name)
 
-    def __init__(self,dbname,*args,**kwargs):
-        super(Database,self).__init__(dbname)
-        self.name = dbname
 
 def requires_config(func):
     @wraps(func)
@@ -51,6 +152,7 @@ def requires_config(func):
         return func(*args,**kwargs)
     return func_wrapper
 
+
 @requires_config
 def get_conn(server, port):
     try:
@@ -60,19 +162,19 @@ def get_conn(server, port):
                                                  server, port)
         else:
             uri = "mongodb://{}:{}".format(server, port)
-        conn = Database(uri)
+        conn = DatabaseClient(uri)
         return conn
     except (AttributeError,ValueError) as e:
         # missing config variables (or invalid), we'll pretend it's a dummy access to mongo
         # (dummy here means there really shouldn't be any call to get_conn()
-        # but mongo is too much tied to the code and needs more work to 
+        # but mongo is too much tied to the code and needs more work to
         # unlink it
         return DummyDatabase()
 
 
 @requires_config
 def get_hub_db_conn():
-    conn = Database(config.HUB_DB_BACKEND["uri"])
+    conn = DatabaseClient(config.HUB_DB_BACKEND["uri"])
     return conn
 
 @requires_config
@@ -144,7 +246,7 @@ def get_target_conn():
                                              config.DATA_TARGET_PORT)
     else:
         uri = "mongodb://{}:{}".format(config.DATA_TARGET_SERVER,config.DATA_TARGET_PORT)
-    conn = Database(uri)
+    conn = DatabaseClient(uri)
     return conn
 
 
@@ -168,7 +270,7 @@ def get_source_fullname(col_name):
     src_dump = get_src_dump()
     # "sources" in config is a list a collection names. src_dump _id is the name of the
     # resource but can have sub-resources with different collection names. We need
-    # to query inner keys upload.job.*.step, which always contains the collection name  
+    # to query inner keys upload.job.*.step, which always contains the collection name
     info = src_dump.find_one({"$where":"function() {if(this.upload) {for(var index in this.upload.jobs) {if(this.upload.jobs[index].step == \"%s\") return this;}}}" % col_name})
     if info:
         name = info["_id"]
@@ -197,7 +299,7 @@ def doc_feeder(collection, step=1000, s=None, e=None, inbatch=False, query=None,
     if isinstance(collection,DocMongoBackend):
         collection = collection.target_collection
     cur = collection.find(query, no_cursor_timeout=True, projection=fields)
-    n = cur.count()
+    n = collection.count_documents(query or {})
     s = s or 0
     e = e or n
     ##logger.info('Retrieving %d documents from database "%s".' % (n, collection.name))
@@ -297,7 +399,7 @@ def id_feeder(col, batch_size=1000, build_cache=True, logger=logging,
        return a doc_feeder for that collection. Valid cache is
        a cache file that is newer than the collection.
        "db" can be "target" or "src".
-       "build_cache" True will build a cache file as _ids are fetched, 
+       "build_cache" True will build a cache file as _ids are fetched,
        if no cache file was found
        "force_use" True will use any existing cache file and won't check whether
        it's valid of not.
@@ -351,7 +453,7 @@ def id_feeder(col, batch_size=1000, build_cache=True, logger=logging,
                 logger.warning("Force building cache file")
                 use_cache = False
             # check size, delete if invalid
-            elif os.path.getsize(cache_file) <= empty_size.get(cache_format,32): 
+            elif os.path.getsize(cache_file) <= empty_size.get(cache_format,32):
                 logger.warning("Cache file exists but is empty, delete it")
                 os.remove(cache_file)
             elif force_use:
@@ -399,12 +501,12 @@ def id_feeder(col, batch_size=1000, build_cache=True, logger=logging,
         else:
             logger.info("Can't build cache, cache not allowed or no cache folder")
             build_cache = False
-        if isinstance(col,Collection):
+        if isinstance(col, PymongoCollection):
             doc_feeder_func = partial(doc_feeder,col, step=batch_size, inbatch=True, fields={"_id":1})
         elif isinstance(col,DocMongoBackend):
             doc_feeder_func = partial(doc_feeder,col.target_collection, step=batch_size, inbatch=True, fields={"_id":1})
         elif isinstance(col,DocESBackend):
-            # get_id_list directly return the _id, wrap it to match other 
+            # get_id_list directly return the _id, wrap it to match other
             # doc_feeder_func returned vals. Also return a batch of id
             def wrap_id():
                 ids = []
@@ -458,7 +560,7 @@ def get_previous_collection(new_id):
     col = get_src_build()
     doc = col.find_one({"_id":new_id})
     assert doc, "No build document found for '%s'" % new_id
-    assert "build_config" in doc, "No build configuration found for document '%s'" % new_id 
+    assert "build_config" in doc, "No build configuration found for document '%s'" % new_id
     assert doc["build_config"]["name"] == doc["build_config"]["_id"]
     confname = doc["build_config"]["name"]
     docs = get_src_build().find({
